#pragma once

#include <boost/asio/spawn.hpp>

#include "json/json.hpp"

#include <deque>


class simple_queue {
	private:
		std::deque<nlohmann::json> queue;
		boost::asio::deadline_timer timer_;

	public:
		simple_queue(boost::asio::io_service& io_service) : timer_(io_service){
			
		}

		void insert(const nlohmann::json& packet) {
			queue.push_back(std::move(packet));
			timer_.cancel();
		}

		nlohmann::json async_pop(boost::asio::yield_context& yieldContext) {
			if (queue.empty()) {
				// Need to wait for an insertion
				timer_.expires_at(boost::posix_time::pos_infin);
				boost::system::error_code ignored_ec;
				timer_.async_wait(yieldContext[ignored_ec]);
			}

			// Now there is something in the queue
			nlohmann::json obj = queue.front();
			queue.pop_front();
			return std::move(obj);
		}
};